'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var validators = require('vuelidate/lib/validators');

var FunctionsMixin = {
  methods: {
    f: function f(val) {
      return typeof val === 'function' ? val() : val;
    },
    // count this.f() for all passed props in an Object
    functions: function functions(obj) {
      var this$1 = this;
      if ( obj === void 0 ) obj = {};

      var props = {};
      for (var i = 0, list = Object.entries(obj); i < list.length; i += 1) {
        var ref = list[i];
        var key = ref[0];
        var val = ref[1];

        props[key] = this$1.f(val);
      }
      return props;
    },
    // count this.f() for all passed props in an Array
    functionsArray: function functionsArray(arr) {
      var this$1 = this;
      if ( arr === void 0 ) arr = [];

      return arr.map(function (val) {
        return this$1.f(val);
      });
    }
  }
};

var FormicsButtons = { render: function () {
    var _vm = this;var _h = _vm.$createElement;var _c = _vm._self._c || _h;return _c(_vm.layout, { tag: "component", attrs: { "slot": "buttons", "schema": _vm.computedButtons, "custom": _vm.custom }, slot: "buttons" }, _vm._l(_vm.computedButtons, function (button, key) {
      return _c('formics-button', _vm._b({ key: key, attrs: { "slot": key, "type": key === 'submit' ? 'submit' : 'button' }, on: { "click": function ($event) {
            _vm.onClick(key);
          } }, slot: key }, 'formics-button', _vm.computedButton(button), false));
    }));
  }, staticRenderFns: [],
  mixins: [FunctionsMixin],
  props: {
    settings: Object,
    layout: String,
    buttons: {
      type: Object,
      default: function default$1() {
        return {
          submit: {
            label: 'submit'
          }
        };
      }
    },
    custom: {
      type: Object,
      default: function default$2() {
        return {};
      }
    }
  },
  computed: {
    computedButtons: function computedButtons() {
      var buttons = {};
      Object.entries(this.buttons).forEach(function (ref) {
        var key = ref[0];
        var val = ref[1];

        if (typeof val === 'string') {
          buttons[key] = {
            label: val
          };
        } else {
          buttons[key] = val;
        }
      });
      return buttons;
    }
  },
  methods: {
    computedButton: function computedButton(button) {
      var this$1 = this;

      var obj = {};
      Object.entries(button).forEach(function (ref) {
        var key = ref[0];
        var val = ref[1];

        // we change `custom` to `customProp` so that we can use `custom` as a field property exposed to templates
        if (key !== 'custom') {
          obj[key] = this$1.f(val);
        } else {
          obj.customProp = this$1.f(val) || {};
        }
      });
      return obj;
    },
    onClick: function onClick(name) {
      this.$emit('event', {
        name: name
      });
    }
  }
};

var CheckboxMixin = {
  methods: {
    // checks if checkbox key should be disabled when the limit is hit
    checkboxLimit: function checkboxLimit(option) {
      if (this.limit) {
        console.log('limit: ' + this.limit);
        console.log('length: ' + this.fieldModel.length);
        console.log('fieldModel: ' + JSON.stringify(this.fieldModel));
        console.log('comparison: ' + this.fieldModel.length >= this.limit);
        console.log('indexOf: ' + this.fieldModel.indexOf(option.value));
        console.log('value:' + option.value);
        console.log('overall:' + this.fieldModel.length >= this.limit && this.fieldModel.indexOf(option.value) === -1);
        return this.fieldModel.length >= this.limit && this.fieldModel.indexOf(option.value) === -1;
      }
    }
  }
};

var CustomMixin = {
  props: {
    // we're naming this customProp so that we can save custom name for templates
    customProp: {
      type: Object,
      default: function default$1() {
        return {};
      }
    }
  },
  computed: {
    // custom options for particular field
    custom: function custom() {
      var this$1 = this;

      var custom = {};
      if (this.root) {
        // custom options for single element
        custom = Object.assign({}, this.defaults, this.functions(this.customProp));
      } else {
        // custom options for multiple elements
        Object.entries(this.defaults || {}).forEach(function (ref) {
          var key = ref[0];
          var val = ref[1];

          custom[key] = Object.assign({}, val, this$1.functions(this$1.customProp[key]));
        });
      }
      return custom;
    }
  }
};

var EmitMixin = {
  props: {
    commit: String,
    dispatch: String,
    event: {
      type: String,
      default: 'change'
    }
  },
  methods: {
    emit: function emit() {
      if (this.$store) {
        this.commitMutation();
        this.dispatchAction();
      }
      if (this.name !== 'form') {
        this.emitEvent(this.event);
      }
      // i guess at some point i decided that groups don't use that
      if (this.isField) {
        this.emitEvent(this.settings.event);
      }
      this.$emit('input', this.fieldModel);
    },
    commitMutation: function commitMutation() {
      if (this.commit) {
        this.$store.commit(this.commit, this.fieldModel);
      }
    },
    dispatchAction: function dispatchAction() {
      if (this.dispatch) {
        this.$store.dispatch(this.dispatch, {
          field: this.name,
          model: this.fieldModel
        });
      }
    },
    emitEvent: function emitEvent(event) {
      if (event) {
        this.$emit('event', {
          name: event,
          field: this.name,
          model: this.fieldModel,
          isValid: !this.invalid
        });
      }
    }
  }
};

var ModelMixin = {
  props: {
    model: {
      required: true
    },
    // is that needed?
    mask: {
      type: String,
      default: ''
    },
    limit: Number,
    default: {
      required: false
    }
  },
  data: function data() {
    return {
      fieldModel: this.model
    };
  },
  watch: {
    model: function model(val) {
      this.fieldModel = val;
      // this.updateCheckbox()
      this.$emit('touch', this.name);
    },
    fieldModel: function fieldModel() {
      this.emit(this.type + ': ' + JSON.stringify(this.fieldModel));
    }
  },
  created: function created() {
    this.fieldModel = this.model;
  }
};

var OptionsMixin = {
  props: {
    // we're calling this optionsProp so that we can save options name for templates
    optionsProp: {
      type: [Object, Array],
      default: function default$1() {
        return {};
      }
    }
  },
  watch: {
    // filters out keys that are not used by schema anymore
    // should it be done for select too? probably not because select options are often dynamic / async
    options: function options() {
      var this$1 = this;

      if (this.type === 'checkbox') {
        this.fieldModel = this.options.filter(function (option) { return this$1.fieldModel.indexOf(option.value) > -1; });
      }
    }
  },
  computed: {
    options: function options() {
      var this$1 = this;

      var result = [];
      var options = this.f(this.optionsProp);
      // array
      if (Array.isArray(options)) {
        console.log('LOL ARRAY: ' + JSON.stringify(options));
        var optionsArray = this.functionsArray(options);
        result = optionsArray.map(function (val) {
          // collection
          if (typeof val === 'object') {
            console.log('NYA OBJECT: ' + JSON.stringify(val));
            return Object.assign({}, this$1.functions(val));
          } else {
            console.log('NYA VALUE: ' + JSON.stringify(val));
            return { text: val, value: val };
          }
        });
        // object
      } else {
        console.log('LOL OBJECT: ' + JSON.stringify(options));
        var optionsObject = this.functions(options);
        result = Object.keys(optionsObject).map(function (key) {
          var val = optionsObject[key];
          console.log('optionsProp for label: ' + key + ': ' + val);
          // nested object
          if (typeof val === 'object') {
            return Object.assign({}, this$1.functions(val), { text: key });
          } else {
            return { value: val, text: key };
          }
        });
      }
      return this.functionsArray(result);
    }
  },
  methods: {
    // bindings for iterated fields like radio, checkbox, switch, select
    getOptionBindings: function getOptionBindings(option, text, value) {
      if ( text === void 0 ) text = 'text';
      if ( value === void 0 ) value = 'value';

      var bindings = Object.assign({}, this.custom.field, option);
      console.log('getOptions before: ' + JSON.stringify(bindings));
      if (text !== 'text') {
        bindings[text] = bindings.text;
        delete bindings.text;
      }
      if (value !== 'value') {
        bindings[value] = bindings.value;
        delete bindings.value;
      }
      console.log('getOptions after: ' + JSON.stringify(bindings));
      return bindings;
    }
  }
};

var PasswordMixin = {
  data: function data() {
    return {
      showPassword: false
    };
  },
  computed: {
    // dynamically change type between password and text for password inputs
    getInputType: function getInputType() {
      if (this.type === 'password') {
        return this.showPassword ? 'text' : 'password';
      } else {
        return this.type;
      }
    }
  }
};

var TextsMixin = {
  props: {
    placeholder: String,
    label: String,
    hint: String
  },
  computed: {
    getLabel: function getLabel() {
      return this.getText('labels', this.label || this.name);
    },
    getPlaceholder: function getPlaceholder() {
      return this.placeholder ? this.getText('placeholders', this.placeholder) : '';
    },
    getHint: function getHint() {
      return this.hint ? this.getText('hint', this.hint) : '';
    }
  },
  methods: {
    getText: function getText(category, text, params) {
      // we declare labels, placeholders and errors separately because this way they're more flexible and reusable across multiple forms
      return this.$i18n ? this.$i18n.t('formics.' + category + '.' + text, params) : text;
    }
  }
};

var ValidationMixin = {
  props: {
    validations: {
      type: Object,
      default: function default$1() {
        return {};
      }
    },
    // is that still needed?
    v: {
      type: Object,
      default: function default$2() {
        return {};
      }
    },
    messages: {
      type: Object,
      default: function default$3() {
        return {};
      }
    }
  },
  computed: {
    invalid: function invalid() {
      return typeof this.$v === 'undefined' ? false : this.$v.$invalid;
    }
  },
  watch: {
    $v: function $v(val) {
      this.$emit('touch', this.name);
    }
  },
  methods: {
    getErrors: function getErrors(v) {
      var this$1 = this;

      var arr = [];
      // are there any validation rules
      if (typeof v === 'object') {
        // is any validator invalid
        if (v.$error) {
          for (var i = 0, list = Object.entries(v.$params); i < list.length; i += 1) {
            // is validator invalid and not a promise
            var ref = list[i];
            var validator = ref[0];
            var params = ref[1];

            if (!v[validator] && !v.$pending) {
              if (this$1.$i18n) {
                arr.push(this$1.$i18n.t('formics.validation.' + validator, params));
              } else {
                arr.push(this$1.messages[validator] || 'error message missing for key: ' + validator);
              }
            }
          }
        }
      }
      // return array of errors or empty array
      return arr;
    }
  }
};

var FormicsMixin = {
  mixins: [CheckboxMixin, CustomMixin, EmitMixin, FunctionsMixin, ModelMixin, OptionsMixin, PasswordMixin, TextsMixin, ValidationMixin],
  props: {
    field: {
      type: Object,
      default: function default$1() {
        return {};
      }
    },
    settings: Object,
    components: {
      type: Object,
      default: function default$2() {
        return {};
      }
    },
    schema: Object,
    disabled: Boolean,
    required: Boolean,
    wrapper: {
      type: String,
      default: 'div'
    },
    layout: [String, Object],
    state: String,
    name: {
      type: String,
      required: true
    },
    type: String,
    i18n: Array
  },
  computed: {
    isField: function isField() {
      return this.type !== 'group';
    }
  },
  methods: {
    isActive: function isActive(active) {
      return typeof active === 'undefined' ? true : this.f(active);
    },
    areEqual: function areEqual(left, right) {
      return JSON.stringify(left) === JSON.stringify(right);
    }
  }
};

var FormicsTypes = {};

var FormicsGroup = { render: function () {
    var _vm = this;var _h = _vm.$createElement;var _c = _vm._self._c || _h;return _c(_vm.layout || _vm.groupLayout, { tag: "component", attrs: { "schema": _vm.groupSchema, "custom": _vm.custom } }, [_vm._l(_vm.groupSchema, function (field, key) {
      return _vm.isActive(field) ? [field.type === 'group' ? [_c('formics-group', _vm._b({ key: key, attrs: { "slot": key, "name": key, "model": _vm.groupModel[key], "v": _vm.$v.groupModel[key] || {}, "messages": _vm.messages, "settings": _vm.settings, "validations": _vm.groupValidations[key] || {}, "schema": _vm.groupSchema[key].schema, "is-valid": _vm.isValid, "group-layout": "settings.group.layout" }, on: { "input": function (val) { return _vm.set(key, val); }, "touch": _vm.touch, "event": _vm.passEvent, "validation": _vm.updateGroupValidations }, slot: key }, 'formics-group', field, false))] : [_c(field.component, _vm._b({ key: key, tag: "component", attrs: { "slot": key, "name": key, "model": _vm.groupModel[key], "v": _vm.$v.groupModel[key] || {}, "messages": _vm.messages, "settings": _vm.settings }, on: { "input": function (val) { return _vm.set(key, val); }, "touch": _vm.touch, "event": _vm.passEvent }, slot: key }, 'component', _vm.computedField(key), false))]] : _vm._e();
    })], 2);
  }, staticRenderFns: [],
  name: 'FormicsGroup',
  mixins: [FormicsMixin],
  props: {
    isValid: Boolean,
    default: false,
    groupLayout: {
      type: String,
      default: 'FormicsGroupLayout'
    }
  },
  data: function data() {
    return {
      groupModel: {},
      groupSchema: {},
      groupValidations: {},
      oldSchema: {}
    };
  },
  created: function created() {
    this.updateSchema();
    this.updateModel(this.model);
    this.$emit('input', this.groupModel);
  },
  watch: {
    schema: {
      handler: function handler(val, oldVal) {
        this.updateSchema();
        this.updateModel(this.groupModel);
        this.$emit('input', this.groupModel);
      },
      deep: true
    },
    groupSchema: {
      handler: function handler(val, oldVal) {
        this.oldSchema = oldVal;
        this.updateValidations('watcher schema');
      },
      deep: true
    },
    model: {
      handler: function handler(val) {
        if (!this.areEqual(val, this.groupModel)) {
          this.updateModel(val);
          this.$emit('input', this.groupModel);
        }
      },
      deep: true
    },
    $v: {
      handler: function handler(val) {
        this.$emit('updateValidationState', this.$v);
      },
      deep: true
    },
    // this is a workaround because directly watching or computing this.$v.groupModel.$invalid
    // results with SSR issues (different errors amount on the server and client)
    groupModel: {
      handler: function handler(val) {
        // only main group should emit updateIsValid when it changes locally
        if (this.name === 'form' && this.isValid !== !this.$v.groupModel.$invalid) {
          this.$emit('update', {
            property: 'isValid',
            payload: !this.$v.groupModel.$invalid
          });
        }
        this.$emit('input', this.groupModel);
      },
      deep: true
    }
  },
  methods: {
    // one of child fields got updated
    set: function set(key, newModel) {
      var oldModel = this.groupModel[key];
      if (!this.areEqual(newModel, oldModel)) {
        this.$set(this.groupModel, key, newModel);
        this.emit('set');
      }
    },
    // touch event for vuelidate
    touch: function touch(name) {
      if (this.$v) {
        if (this.$v.groupModel[name]) {
          this.$v.groupModel[name].$touch();
          this.$emit('touch', name);
        }
      }
    },
    updateSchema: function updateSchema() {
      var localSchema = {};
      Object.entries(this.schema).forEach(function (ref) {
        var key = ref[0];
        var val = ref[1];

        var obj = typeof val === 'string' ? { type: val } : val; // for fields defined as string
        obj.type = obj.type || 'text';
        obj.component = FormicsTypes[obj.type].component;
        // update default model
        if (typeof obj.default !== typeof FormicsTypes[obj.type].default) {
          obj.default = FormicsTypes[obj.type].default;
        }
        if (typeof obj.model === 'undefined') {
          obj.model = obj.default;
        }
        localSchema[key] = obj;
      });
      this.groupSchema = localSchema;
    },
    // separate loop makes sure that the unused keys get removed from this.model
    updateModel: function updateModel(model) {
      var localModel = {};
      Object.entries(this.groupSchema).forEach(function (ref) {
        var key = ref[0];
        var val = ref[1];

        localModel[key] = model.hasOwnProperty(key) ? model[key] : val.default;
      });
      this.groupModel = localModel;
    },
    // separate loop makes sure that the unused keys get removed
    updateValidations: function updateValidations(val) {
      var this$1 = this;

      // let localValidations = Object.keys(this.groupValidations).length > 0 ? this.groupValidations : this.validations
      var localValidations = {};
      Object.entries(this.groupSchema).forEach(function (ref) {
        var key = ref[0];
        var val = ref[1];

        if (this$1.validations.hasOwnProperty(key) || val.validators) {
          if (this$1.f(val.required)) {
            val.validators.required = validators.required;
          }
          localValidations[key] = Object.assign({}, this$1.validations[key] || {}, val.validators || {});
        }
      });
      this.groupValidations = localValidations;
      this.$emit('update', {
        property: 'validations',
        payload: this.groupValidations
      });
    },
    updateGroupValidations: function updateGroupValidations(val) {
      this.$set(this.groupValidations, this.name, val);
    },
    // is field invalid (for errors slot)
    isInvalid: function isInvalid(key) {
      if (this.$v.groupModel.hasOwnProperty(key)) {
        return this.$v.groupModel[key].$invalid;
      } else {
        return false;
      }
    },
    // pass the events further to the top
    passEvent: function passEvent(event) {
      if (this.name !== 'form') {
        event.field = this.name + '.' + event.field;
      }
      if (event.field !== 'form') {
        console.log('event.field = ' + event.field);
        this.$emit('event', event);
      }
    },
    // prepare field props for v-bind
    computedField: function computedField(field) {
      var this$1 = this;

      var localField = {};
      Object.entries(this.groupSchema[field]).forEach(function (ref) {
        var key = ref[0];
        var val = ref[1];

        // we change `custom` to `customProp` so that we can use `custom` as a field property exposed to templates
        if (key === 'custom') {
          localField.customProp = this$1.f(val) || {};
        } else if (key === 'options') {
          localField.optionsProp = this$1.f(val) || [];
        } else {
          localField[key] = this$1.f(val);
        }
      });
      return localField;
    }
  },
  // vuelidate magic happens here
  validations: function validations() {
    return {
      groupModel: this.groupValidations || {}
    };
  }
};

var FormicsForm = { render: function () {
    var _vm = this;var _h = _vm.$createElement;var _c = _vm._self._c || _h;return _c('form', { on: { "submit": function ($event) {
          $event.preventDefault();_vm.submit($event);
        } } }, [_c('formics-group', { key: "form", attrs: { "slot": "fields", "name": "form", "type": "group", "model": _vm.model, "settings": _vm.computedSettings, "validations": _vm.validations, "schema": _vm.schema, "is-valid": _vm.isValid, "group-layout": "FormicsFormLayout" }, on: { "input": _vm.onInput, "update": function (val) { return _vm.emitUpdate(val); }, "event": _vm.passEvent }, slot: "fields" }), _vm._v(" "), _c('formics-buttons', { attrs: { "layout": _vm.computedSettings.buttons.layout || 'FormicsButtonsLayout', "buttons": _vm.buttons, "custom": _vm.computedSettings.buttons.custom || {} }, on: { "event": _vm.passEvent } })], 1);
  }, staticRenderFns: [],
  name: 'FormicsForm',
  components: {
    FormicsButtons: FormicsButtons,
    FormicsGroup: FormicsGroup
  },
  props: {
    settings: {
      type: Object,
      default: function default$1() {
        return {};
      }
    },
    schema: {
      type: Object,
      required: true
    },
    model: {
      type: Object,
      default: function default$2() {
        return {};
      }
    },
    buttons: {
      type: Object,
      default: function default$3() {
        return {};
      }
    },
    // optional vuelidate schema
    validations: {
      type: Object,
      default: function default$4() {
        return {};
      }
    },
    // optional read-only property
    isValid: {
      type: Boolean,
      default: false
    }
  },
  computed: {
    // default values for all settings
    computedSettings: function computedSettings() {
      var settings = this.settings;
      var arr = ['form', 'group', 'fields', 'buttons', 'messages'];
      arr.forEach(function (key) {
        settings[key] = settings[key] || {};
      });
      return settings;
    }
  },
  methods: {
    // emit event back to component if a button was clicked with no callback provided
    // argument is the key of in the buttons object
    // update model in the parent
    onInput: function onInput(val) {
      if (JSON.stringify(val) !== JSON.stringify(this.model)) {
        this.$emit('input', val);
      }
    },
    // emit custom update event for `.sync` purposes
    emitUpdate: function emitUpdate(ref) {
      var property = ref.property;
      var payload = ref.payload;

      this.$emit('update:' + property, payload);
    },
    // pass custom events
    passEvent: function passEvent(event) {
      var name = event.name;
      delete event.name;
      this.$emit(name, event);
    },
    submit: function submit() {
      console.log('submit');
    }
  }
};

var LayoutMixin = {
  props: {
    schema: {
      type: Object,
      default: function default$1() {
        return {};
      }
    },
    custom: {
      type: Object,
      default: function default$2() {
        return {};
      }
    }
  },
  methods: {
    f: function f(val) {
      return typeof val === 'function' ? val() : val;
    }
  }
};

var ButtonMixin = {
  mixins: [CustomMixin, FunctionsMixin],
  props: {
    label: String,
    callback: Function,
    disabled: {
      required: false
    }
  },
  data: function data() {
    return {
      root: true,
      defaults: {}
    };
  },
  methods: {
    // emit an event back to FormicsForm or trigger the callback function if provided
    action: function action() {
      typeof this.callback === 'function' ? this.callback() : this.$emit('click');
    }
  }
};

// Install the components
var install = function (Vue, Components) {
  Vue.component('FormicsForm', FormicsForm);

  Object.keys(Components).forEach(function (key) {
    Vue.component(key, Components[key]);
  });
};

var Formics = {
  install: install
};

// Auto-install
if (typeof window !== 'undefined' && window.Vue) {
  window.Vue.use(Formics);
}

exports.Formics = Formics;
exports.ButtonMixin = ButtonMixin;
exports.LayoutMixin = LayoutMixin;
exports.FormicsMixin = FormicsMixin;
exports.FormicsTypes = FormicsTypes;
exports.FormicsForm = FormicsForm;
