# Formics {docsify-ignore-all}

<img src="https://vignette.wikia.nocookie.net/ansible/images/3/36/Formic_Worker_Comic.jpg" class="right">Formics is an alien, insect-like species from [The Ender's Game](https://en.wikipedia.org/wiki/Ender%27s_Game) by Orson Scott Card. Their workers and warriors travel through the galaxy in search of new planets to live on and new resources to extract. There may be thousands of them spread around the world, but they don't really exist as individual beings. Each swarm lives as one organism, a hive-mind telepathically controlled by its queen. What the queen thinks of, happens in an instant. Just how the code should instantly do what the developer wants it to.

#### Flexible form generator for Vue.js

That's exactly how the forms in Formics are structured. Sharing one mind - a data object that manages its network of components. Passing orders via props and getting reports back through custom events. With every part of the system following the developer's grand master plan. 

Look at your code and tell me, how many forms do you have in your apps. How many fields with their inputs, labels, validation bindings, error messages and complex HTML layout. How many potential points of failure and how much rewriting if you decide to make a small design change. How many wasted working hours that you could spend in a better way.

You shouldn't need to bother with that. Ultimately, what you want is to write your application with its cool features that will make the world a better place. And it just so happens, make you insanely rich as a side effect.

Let Formics take care of the dirty work for you.

#### Powerful, dynamic schema

With Formics, you manage your form through a dynamic schema object. All changes you make to this object have instant effect on the form. Field settings can be defined both as a literal (for example Boolean, String or Array) or a function that returns it, which makes it easy to change your forms based on their content or other data.

You can look for changes in the model based on events, watchers or callbacks. Fields can also dispatch their own Vuex actions or mutations. Labels, placeholders and hints can be localized with vue-18n under the hood.

#### Modular templates for multiple UI libraries

Formics doesn't come with template components in its main module. Instead, it works with separate NPM modules with templates for popular UI libraries. You can improve them with PRs or just fork them and rewrite to better suit your needs, without touching the rest of the library. Alternatively, you can just write custom fields or a whole set of own templates. There are no restrictions on how many field types you have to include in your bundle. You can choose only the ones you actually need.

Each template can have its own customized settings that can be dynamically used in schema on field, group or form level. You can also freely customize the way you want your form, group or buttons layout to look like and make them dynamic with your schema.

#### Validation with Vuelidate

Under the hood, Formics works with [Vuelidate](https://monterail.github.io), a battle-tested validation solution with well-known and trustworthy maintainers - organizers of VueConf 2017 and one of key Vue.js sponsors. 

You can reuse your existing Vuelidate objects when you opt in, but you can also opt out of Formics and still reuse your validation schema with a custom form solution. You can also mix validation rules into your main schema object, which will let you dynamically generate validation together with changes in the schema. You can even use both at the same time for rapid prototyping or transition period.
