# Component

### Template

From the level of your page components, you only use the main FormicsForm component. All the rest including layouts, fields and buttons become its children.

Here's a full list of all available attributes. The only actually required one is `schema`, but usually you'll want to handle at least `v-model` and `@submit` event, just like you would with any other form.

```html
<template>
  <FormicsForm 
    :schema="schema"
    :settings="settings" 
    :buttons="buttons" 
    :validations="validations" 
    :isValid.sync="isValid"
    :v.sync="v"
    v-model="model" 
    @submit="onSubmit" 
    @change="onChange"
    @customEvent="onCustomEvent"
  />
</template>
```

You probably already noticed that only some props use `.sync` modifier. Together with `model` and events, these properties serve as a source of feedback from Formics to your component about what happens inside.

### Form Properties

##### Buttons

- **Type**: `Object`
- **Required**: `false`
- **Mode**: Write-only
- **Description**: A set of buttons that you want to use in your form. If you don't provide it, Formics shows a standard `submit` button by default.
- **Read more**: [Buttons](buttons.md)

##### Model

- **Type**: `Object`
- **Required**: `false`
- **Mode**: `v-model`
- **Description**: Model object with values of all fields across the form. The values you provide get merged with default values in the schema. It comes handy in particular situations, such as overwriting the default model with data from an api call. The model is synchronized with field values - whenever the form is updated, the model will get the new values too.
- **Read more**: [Model](model.md)

##### Schema

- **Type**: `Object`
- **Required**: `false`
- **Mode**: Write-only
- **Description**: Configuration for the Formics library. It gets merged with default configuration, so you only need to provide data you want to be changed.
- **Read more**: [Schema](schema.md)

##### Settings

- **Type**: `Object`
- **Required**: `false`
- **Mode**: Write-only
- **Description**: Configuration for the Formics library. It gets merged with default configuration, so you only need to provide data you want to be changed.
- **Read more**: [Settings](settings.md)

### Validation Properties

Validation props are a bit specific, because they provide `.sync` support for some features you hardly ever need.

##### Validations

- **Type**: `Object`
- **Required**: `false`
- **Mode**: `:prop`
- **Description**: Vuelidate's `validations` object. Use it if you already use Vuelidate and want to quickly reuse your configuration in Formics. The values you provide get merged with validators in the schema.
- **Read more**: [Validation](validation.md)

##### isValid

- **Type**: `Boolean`
- **Required**: `false`
- **Mode**: `.sync`
- **Description**: True when your form is valid, false when it isn't. Comes handy at times.

##### v

- **Type**: `Object`
- **Required**: `false`
- **Mode**: `.sync`
- **Description**: Vuelidate's `$v` object for whole model, reactively updated with all changes. You usually don't need to use it, but sometimes you have your reasons, so here it is.
- **Read more**: [Validation](validation.md)