
### Default events

There are two default events that you can listen for. You can also create custom events.

##### @input

- **Description**: Input event used by `v-model` binding under the hood.
- **Argument**: `Object` containing whole form's model

##### @submit
- **Description**: Traditional submit event to send the form data to the server.
- **Argument**: `Object` containing whole form's model

```js
    methods: {
      submit (model) {
        console.log('Submitted value: ' + JSON.stringify(model))
        // Submitted value: {"username": "gusto", "password": "gusto-is-awesome"}
        console.log('Submitted value: ' + JSON.stringify(this.model))
        // Submitted value: {"username": "gusto", "password": "gusto-is-awesome"}
      }
    }
```

##### @change
- **Description**: Emitted when any of the field values is updated, with data about the updated field in particular.
- **Argument**: `Object` with two keys:
  - field: `String` name of the field
  - path: `String` path of the field
  - model: Value of the updated field
  - isValid: `Boolean` flag, true when the field fullfills all validation rules
  - v: `Object` copy of Vuelidate's `$v` value for the field

```js
    methods: {
      submit ({ field, model }) {
        console.log('Updated field: ' + field
        // Updated field: password
        console.log('New value: ' + JSON.stringify(model))
        // New value: gusto-is-awesome
      }
    }
```

If the field belongs to a group, the path string will be prefixed by its name. For example if a field `town` belongs to a group of fields `address`, the path will be provided as `address.town`.

### Custom events

### Vuex events