# Fields

### Definition

Each field in the form is represented by a separate `Object` in the schema. The key of this object serves quite a few purposes:

- the `name` attribute of the field
- default label value for `label` property is not provided
- the key for the field's value in `form.model` object

Alternatively, field can be defined as a `String`. The following field definition:

```js
username: 'text'
```

will be transpiled to:

```js
username: {
  type: 'text'
}
```

As you can see, `type` is the only required property of the field.

### Properties

Each of the properties except for the `type` can be alternatively represented as a `Function` returning appropriate value. This allows you to make the field's schema dynamic, for example when you want to provide the list of options for the select field from the server call.

Remember to use fat arrow syntax to bind the data context of the component in which the schema is defined.

```js
options: () => this.options
```

##### active

- **Type**: `Boolean`
- **Required**: `false`
- **Default**: `true`
- **Description**: Field with `active` property set to `false` is not only removed from the DOM but also from the `form.model` object. When you set it back to `true`, its value will be back to default.

##### commit

- **Type**: `String`
- **Required**: `false`
- **Description**: Name of the Vuex mutation to be commited when the field value is updated. Remember to provide a proper module namespace.
- **Read more**: [Events](events.md)

##### custom

- **Type**: `Object`
- **Required**: `false`
- **Description**: A set of custom properties to be bound to particular elements of the field template.

##### default

- **Type**: `Any`
- **Required**: `false`
- **Description**: Use it ff you want the schema to provide a specific default value for the field, other than the default value for the field's `type`. It's used only in case the value is not already provided in `form.model`.
- **Read more**: [Model](model.md)

##### dispatch

- **Type**: `String`
- **Required**: `false`
- **Description**: Name of the Vuex action to be dispatched when the field value is updated. Remember to provide a proper module namespace.
- **Read more**: [Events](events.md)

##### event

- **Type**: `String`
- **Required**: `false`
- **Description**: Name of a custom event to be emitted when the field value is updated.
- **Read more**: [Events](events.md)

##### hint

- **Type**: `String`
- **Required**: `false`
- **Description**: Hint is a message with a short information about the field. It can for example suggest a way to fill it or explain the purpose of the question asked. In some templates (especially those based on material design) hints will not be visible in error state.
- **Read more**: [Field Slots](field-slots.md)

##### label

- **Type**: `String`
- **Required**: `false`
- **Description**: Label of the field. If it's not provided, the key of the field object will be used as a default value.
- **Read more**: [Field Slots](field-slots.md)

##### placeholder

- **Type**: `String`
- **Required**: `false`
- **Description**: Placeholder of the field. In some templates (especially those based on material design) placeholders may be replaced by a label, depending on selected custom options.
- **Read more**: [Field Slots](field-slots.md)

##### type

- **Type**: `String`
- **Required**: `true`
- **Description**: Type is used to find default configuration of the field in the `Types` object. Based on the type, the form knows what kind of data structure the field is (for example `Boolean`), what should be its default value and what component should be used to display it.

##### validators

- **Type**: `Object`
- **Required**: `false`
- **Description**: List of vuelidate validators with their parameters to be merged into `validations` option of the `form` object.
- **Read more**: [Validation](validation.md)

##### wrapper

- **Type**: `String`
- **Required**: `false`
- **Description**: String name of the component to be used as a wrapper for the field, in case the field's template handles such a property. An example can be a datepicker that can be displayed either below the input field or in a modal.

### Custom Properties

fdsafds