# Groups

### Definition

Sometimes you want to group your fields in an object so that it can be handled on the server together. This is especially useful when you work with NOSQL databases such as MongoDB.

Additionally you may want to provide separate styling for a group of fields, with its own layout and subheader.

This is what the schema groups are for. You can create a group by writing a following schema:

```js
schema: {
  address: {
    type: 'group',
    schema: {
      street: 'text',
      town: 'text',
      postcode: 'text',
      country: 'text
    }
  }
}
```
The created model would be:
```js
console.log(JSON.stringify(this.form.model))
// { "address": { street: "", "town": "", "postcode": "", "country": "" } }
```

### API

##### type

- **Type**: `String`
- **Required**: `true`
- **Description**: To create a group, use `type: 'group'`.

##### schema

- **Type**: `Object`
- **Required**: `true`
- **Description**: Schema for a group object follows the same rules as root schema of the form.

##### layout

- **Type**: `String|Object`
- **Required**: `false`
- **Default**: `FormicsGroupLayout`
- **Description**: A component object that will serve as a group layout or name of the component in case it's globally registered.

##### custom

- **Type**: `Object`
- **Required**: `false`
- **Description**: A set of custom properties to be passed to group's layout.
- **Read more**: [Custom Properties](custom.md)
