# Model

### Definition

The `form.model` object represents the overall model of the form. Each field is represented by the key adequate to its key in the schema. Similarily schema groups become objects with the key being the key of the group. Here's an example of a model with an `address` group:

```js
{
  firstName: 'Sherlock',
  lastName: 'Holmes',
  address: {
    street: '221b Baker St',
    town: 'London',
    postcode: 'NW1 6XE',
    country: 'United Kingdom'
  }
}
```

What's important, it's not required for the form object to provide a model. In such a case, it will be automatically created by Formics based on schema, thanks to the model merging system.

### Merging system

Formics lets you to provide default model values in two ways. Either you define them in your schema with `default` prop or in the `form.model` object.

Values from the `model` object get merged with `default` values from the schema. The merging system looks for the value in a particular order of places and stops the cycle once it's found, even if the value is empty.

<ol>
  <li>the value of field's key in model object</li>
  <li>the `default` property for particular field provided in the schema</li>
  <li>the `default` property adequate to the field type in Types object</li>
</ol>

This means that you can't use `default` prop in schema to reactively change the field's model value. The `default` value is only meant to provide the initial value of the field when the field gets created or gets active again.

##### Unused fields

The merging system is designed to automatically clear the model from all values that are not used by the schema. These values don't get cached either. That means that if a field doesn't exist anymore in the schema or if it exists but is set to inactive, its value will be lost.

### Updating the model

You can update your model at any given moment, either as a whole model, group or individual field. Formics will grab the changes and apply them to particular fields.

Remember about the caveats of Vue reactivity system. Unless you use `2.x-next` versions of Vue (which are not available yet anyways), be sure to use `this.$set` for your updates.

For example, how about updating the whole form with values from api call:

```js
methods: {
  updateModel () {
    axios.get(`https://my-awesome-api.com`)
      .then({data} => {
        this.$set(this.form, 'model', data)
      }
  }
}
```

To keep things sane, such updates don't trigger `@change` and custom events which you'd be flooded with otherwise. (?)