# Quick Start

### Install NPM modules

To use Formics, you'll need three NPM modules. Formics, Vuelidate and a template library of your choice. If you want to use your custom templates instead, just install first two packages.

```
npm install formics formics-buefy vuelidate --save
```

### Register Vue plugins

Next thing to do is to register both Formics and Vuelidate. I make an assumption here that you already know how to use Vue plugins and Javascript's modules system.

```js
import Vue from 'vue'
import Vuelidate from 'vuelidate'
import Formics from 'formics'
import * as Components from 'formics-buefy'

Vue.use(Formics, Components)
Vue.use(Vuelidate)
```

Feel free to choose all or just the components you need. You can also mix them with your own ones. The only thing the `Vue.use(Formics)` is responsible for is to globally register its own main component and the passed templates.

If you prefer to register the components localy or even asynchronously with dynamic imports, check out [Custom Installation](installation.md) chapter.

### Write a component

From the level of your pages, you only use the main FormicsForm component. All the rest including layouts, fields and buttons is generated from the props values. There's also a set of [events](events.md) providing feedback from the form that you may want to write listeners for.

Here's a smallest example of how that can look like.

```html
<template>
  <FormicsForm :schema="schema" v-model="model" @submit='submit' />
</template>

<script>
  export default {
    name: 'MyFirstFormicsForm',
    data () {
      return {
        schema: {
          username: 'text',
          password: 'password'
        },
        model: {}
      }
    },
    methods: {
      submit () {
        console.log('Submitted values: ' + this.model)
      }
    }
  }
</script>
```